#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from libsan.host.stratis import Stratis
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.linux import mount, umount, mkdir
from stqe.host.persistent_vars import read_env


def pool_out_of_space():
    errors = []

    pool_name = "pool_out_of_space2"
    fs_size = 512
    loopdev_size = 10400
    fs_name = "fs_fail"
    mount_dir = "/mnt/pool_out_of_space"
    stratis_fs_path = f"/dev/stratis/{pool_name}/fs1"
    no_overprovision = None
    stratis = Stratis()

    try:
        no_overprovision = read_env("fmf_pool_nooverprovision")
    except KeyError:
        pass

    loop_path = create_loopdev(size=loopdev_size)

    if not no_overprovision:
        atomic_run(f"Creating pool: {pool_name}",
                   pool_name=pool_name,
                   blockdevs=loop_path,
                   command=stratis.pool_create,
                   errors=errors
                   )

    else:
        atomic_run(f"Creating pool: {pool_name}",
                   pool_name=pool_name,
                   blockdevs=loop_path,
                   no_overprovision=True,
                   command=stratis.pool_create,
                   errors=errors
                   )

    # for an approximate result in MiB, divide loopdev_size value by 1.049. Then divide
    # result with fs size to get how many fs we should create in order to fill pool.
    # pool always takes ~500MiB when created, so this formula should execute 1 or 2
    # fs create commands that will fail.
    num_of_filesystems = int((loopdev_size / 1.049) / fs_size)
    for fs in range(num_of_filesystems):
        stratis.fs_create(pool_name=pool_name,
                          fs_name=f"fs{fs}",
                          fs_size=f"{fs_size}MiB", )

    atomic_run("Creating FS, this should fail because doesn't have enough space",
               expected_out=['Error: Command failed:', 'No space left on device'],
               expected_ret=1,
               pool_name=pool_name,
               fs_name=fs_name,
               command=stratis.fs_create,
               errors=errors
               )

    atomic_run(f"Creating dir: {mount_dir}",
               new_dir=mount_dir,
               command=mkdir,
               errors=errors
               )

    atomic_run(f"Mounting filesystem: {fs_name}",
               device=stratis_fs_path,
               mountpoint=mount_dir,
               command=mount,
               errors=errors
               )

    # at this point fs snapshot should fail as well because pool has not enough space
    atomic_run("Creating FS snapshot, this should fail because doesn't have enough space",
               expected_ret=1,
               # expected_out="",
               pool_name=pool_name,
               snapshot_name="snap1",
               origin_name="fs1",
               command=stratis.fs_snapshot,
               errors=errors
               )

    atomic_run("Unmounting filesystem",
               mountpoint=mount_dir,
               command=umount,
               errors=errors
               )

    for fs in range(num_of_filesystems):
        atomic_run(f"Destroying filesystem: fs{fs}",
                   pool_name=pool_name,
                   fs_name=f"fs{fs}",
                   command=stratis.fs_destroy,
                   errors=errors
                   )

    atomic_run(f"Destroying pool: {pool_name}",
               pool_name=pool_name,
               command=stratis.pool_destroy,
               errors=errors
               )

    atomic_run("Removing loopdev",
               name=loop_path,
               command=delete_loopdev,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = pool_out_of_space()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.stratis import Stratis
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, read_env


def restart_pool():
    errors = []
    stratis = Stratis()
    pool_name = None
    pool_uuid = None
    unlock_method = None

    try:
        pool_name = read_env("fmf_pool_name")
    except KeyError:
        pass
    if not pool_name:
        try:
            pool_name = read_var("STRATIS_POOL")
            pool_uuid = read_var("STRATIS_POOL_UUID")
        except KeyError:
            pass
    if not pool_uuid:
        pool_uuid = stratis.get_pool_uuid(pool_name=pool_name)
    try:
        unlock_method = read_env("fmf_unlock_method")
    except KeyError:
        pass

    atomic_run("Stopping stratis pool %s." % pool_name,
               command=stratis.pool_stop,
               pool_name=pool_name,
               errors=errors)

    atomic_run("Starting stratis pool %s." % pool_name,
               command=stratis.pool_start,
               unlock_method=unlock_method,
               pool_uuid=pool_uuid,
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = restart_pool()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.stratis import Stratis
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, clean_var, read_env, write_var


def destroy_pool():
    errors = []
    stratis = Stratis()

    id = ""
    try:
        id = "_" + str(read_env("fmf_pool_id"))
    except KeyError:
        pass
    pool_name = read_var("STRATIS_POOL%s" % id)

    if "single" in read_env("fmf_name"):
        available_devices = read_var("STRATIS_AVAILABLE_DEVICES")
        write_var({"STRATIS_DEVICE": available_devices})
        free = read_var("STRATIS_FREE")
        if free:
            clean_var("STRATIS_FREE")

    atomic_run("Destroying stratis pool %s." % pool_name,
               command=stratis.pool_destroy,
               pool_name=pool_name,
               errors=errors)

    atomic_run("Cleaning var STRATIS_POOL%s" % id,
               command=clean_var,
               var='STRATIS_POOL%s' % id,
               errors=errors)

    atomic_run("Cleaning var STRATIS_POOL_UUID",
               command=clean_var,
               var='STRATIS_POOL_UUID',
               errors=errors)

    return errors


if __name__ == "__main__":
    errs = destroy_pool()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from libsan.host.linux import wait_udev
from stqe.host.atomic_run import atomic_run, parse_ret


def cleanup_multipath():
    print("INFO: Removing multipath.")
    errors = []

    ret, multipath = atomic_run(message="Getting mpath device",
                                command=run,
                                return_output=True,
                                cmd="multipath -l -v 1",
                                errors=errors)

    cmd = "-F"
    if multipath is not None:
        cmd = "-f %s" % multipath
    for mpath in cmd.splitlines():
        if '-' not in mpath:
            mpath = '-f ' + mpath
        atomic_run(message="Removing mpath device %s" % multipath,
                   command=run,
                   cmd="multipath %s" % mpath,
                   errors=errors)

    # wait for udevadm to settle
    wait_udev()

    return errors


if __name__ == "__main__":
    errs = cleanup_multipath()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.fmf_tools import get_env_args, get_func_from_string
from stqe.host.lvm import get_dmpd_args
from libsan.host.dmpd import DMPD


def dmpd_cli():
    errors = []
    dmpd = DMPD()
    args = get_dmpd_args(dmpd_object=dmpd)
    args.update(get_env_args({"source_vg": str, "source_lv": str, "source_file": str}))
    if "thin_trim" == args["command"]:
        args["data_dev"] = f'{args["data_dev"]}_tdata'
    args['command'] = get_func_from_string(dmpd, args.pop('command'), local_functions=globals())

    atomic_run(errors=errors,
               **args)

    return errors


if __name__ == "__main__":
    errs = dmpd_cli()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
# Author: Bruno Goncalves   <bgoncalv@redhat.com>

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
import libsan.host.scsi as scsi
import libsan.host.mp as mp
import libsan.host.linux as linux
import libsan.host.dt as dt
import libsan.sanmgmt as sanmgmt
import stqe.host.nfs_lock as nfs_lock
import stqe.host.LogChecker as log
import re
import sys
import os
import subprocess
import argparse
from six.moves import range

LOCK_TYPE = "shared"

clean_up_info = {}

lock_obj = None


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)

    # Append time information to command
    date = "date \"+%Y-%m-%d %H:%M:%S\""
    p = subprocess.Popen(date, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, stderr = p.communicate()
    stdout = stdout.decode('ascii', 'ignore')
    stdout = stdout.rstrip("\n")

    print("[%s] %s" % (stdout, string))

    sys.stdout.flush()
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def _clean_up():
    """
    """
    global LOCK_TYPE, clean_up_info, lock_obj

    if not clean_up_info:
        return True

    mpath_names = clean_up_info["mpath_names"]
    if mpath_names:
        for mpath in mpath_names:
            mp.remove_mpath(mpath)
        clean_up_info["mpath_names"] = []

    wwids = clean_up_info["wwids"]
    if wwids:
        for wwid in wwids:
            scsi_ids = scsi.scsi_ids_of_wwid(wwid)
            if not scsi_ids:
                continue
            for scsi_id in scsi_ids:
                scsi_disk = scsi.get_scsi_disk_name(scsi_id)
                _print("INFO: Deleting disk: %s" % scsi_disk)
                scsi.delete_disk(scsi_disk)
            mp.mpath_conf_remove('/blacklist_exception/wwid', wwid)
        clean_up_info["wwids"] = []

    _print("INFO: All disks removed from the server")
    obj_sanmgmt = clean_up_info["obj_sanmgmt"]
    lun_names = clean_up_info["lun_names"]
    if lun_names:
        for lun_name in lun_names:
            _print("INFO: Deleting %s on the array..." % lun_name)
            if not obj_sanmgmt.lun_remove(lun_name):
                _print("WARN: Could not remove TMP LUN: %s" % lun_name)
    if lock_obj:
        lock_obj.release_lock(LOCK_TYPE)
    _print("INFO: Cleaned")
    return True


def execute_test(mpath_name, lun_count, lun_size):
    global LOCK_TYPE, clean_up_info, lock_obj

    _print("INFO: Using %s as base device" % mpath_name)
    _print("INFO: Will create %d new LUNs for this test" % lun_count)

    mp_info = mp.multipath_query_all(mpath_name)
    if not mp_info:
        _print("FAIL: Got NULL from query_mp_info(%s)" % mpath_name)
        return False

    obj_sanmgmt = sanmgmt.create_sanmgmt_for_mpath(mpath_name)
    if not obj_sanmgmt:
        _print("FAIL: Got NULL from create_sanmgmt_for_mpath(%s)" % mpath_name)
        return False

    clean_up_info["obj_sanmgmt"] = obj_sanmgmt

    scsi_disks = mp_info["disk"]

    scsi_host_ids = []
    for scsi_disk in scsi_disks:
        host_id = scsi.scsi_host_of_scsi_name(scsi_disk)
        if host_id and host_id not in scsi_host_ids:
            scsi_host_ids.append(host_id)

    clean_up_info["lun_names"] = []
    clean_up_info["wwids"] = []
    clean_up_info["mpath_names"] = []

    lock_obj = nfs_lock.setup_nfs_lock_for_mpath(mpath_name)
    if lock_obj:
        _print("INFO: Requesting NFS lock (%s)..." % LOCK_TYPE)
        if not lock_obj.request_lock(LOCK_TYPE):
            _print("FAIL: Could not request NFS lock")
            return False

        _print("INFO: Waiting for NFS lock...")
        if not lock_obj.get_lock():
            _print("FAIL: Give up waiting for lock")
            sys.exit(1)
        _print("INFO: Good, got NFS lock. Start testing...")

    for lun_cnt in range(1, lun_count + 1):
        lun_name = obj_sanmgmt.lun_create_and_map(size=lun_size)
        if not lun_name:
            _print("FAIL: Could not create LUN %d for add_new_lun test" % lun_cnt)
            _clean_up()
            return False
        clean_up_info["lun_names"].append(lun_name)

    _print("INFO: All LUNs successfully created")

    # need to rescan hosts
    for host in scsi_host_ids:
        scsi.rescan_host(host)
    # if adding more luns we should wait more time
    linux.sleep(6 * lun_count)
    linux.wait_udev()

    if "lun_names" not in clean_up_info:
        _print("FAIL: No LUNs created for test")
        _clean_up()
        return False

    for lun_name in clean_up_info["lun_names"]:
        scsi_ids = obj_sanmgmt.scsi_id_of_lun(lun_name=lun_name)
        if not scsi_ids:
            _print("FAIL: Failed to find out the SCSI ID for newly mapped LUN %s" % lun_name)
            run("ls /dev/sd*")
            print(scsi.query_all_scsi_disks())
            _clean_up()
            return False

        _print("INFO: New SCSI disks created:")
        for _ids in scsi_ids:
            _print("    %s - %s" % (_ids, scsi.get_scsi_disk_name(_ids)))

        # All disks should have the same wwid
        scsi_disk = scsi.get_scsi_disk_name(scsi_ids[0])
        wwid = scsi.wwid_of_disk(scsi_disk)
        if not wwid:
            _print("FAIL: Could not find WWID for %s" % scsi_disk)
            _clean_up()
            return False

        clean_up_info["wwids"].append(wwid)

        _print("INFO: New disks have WWID: %s" % wwid)
        if not mp.mpath_conf_add('/blacklist_exceptions/wwid', wwid):
            _print("FAIL: Cannot enable multipath for WWID: %s" % wwid)
            _clean_up()
            return False

        mp.multipath_reload_conf()
        linux.wait_udev()

        new_mpath = mp.mpath_name_of_wwid(wwid)
        if not new_mpath:
            _print("FAIL: Could not find mpath for WWID %s" % wwid)
            _clean_up()
            return False
        clean_up_info["mpath_names"].append(new_mpath)

        _print("INFO: Multipath enabled on tmp LUN: %s" % new_mpath)

        new_mp_info = mp.multipath_query_all(new_mpath)
        if not new_mp_info:
            _print("FAIL: Could not query mp info for %s" % new_mpath)
            _clean_up()
            return False

        lun_info = obj_sanmgmt.lun_info(lun_name)

        all_scsi_ids_info = scsi.query_all_scsi_disks()
        array_size = int(lun_info["size"])
        for scsi_id in scsi_ids:
            disk_size = int(all_scsi_ids_info[scsi_id]["size"])
            if all_scsi_ids_info[scsi_id]["vendor"] == 'EQLOGIC':
                tolerance = int(array_size * 0.01)
                # check if disk is within tolerance level
                if (disk_size // array_size) < tolerance:
                    array_size = disk_size
            if disk_size != array_size:
                scsi_name = scsi.get_scsi_disk_name(scsi_id)
                _print("FAIL: newly created Disk: %s is holding size: '%s', while on storage array it is: '%s'" % (
                    scsi_name, disk_size, array_size))
                _clean_up()
                return False

        # Check if mpath reported size correctly
        mp_size = new_mp_info["size_bytes"]
        if int(mp_size) != int(lun_info["size"]):
            _print("FAIL: newly created mpath: %s is holding size: %s, while on storage array it is: %s" % (
                new_mpath, mp_size, int(lun_info["size"])))
            _clean_up()
            return False

    # end lun_names for
    _print("INFO: All LUNs successfully found by host")

    # Run DT in all LUNs
    dt_pid_2_info = {}
    for mpath_name in clean_up_info["mpath_names"]:
        dt_log_file = "/tmp/dt_log_%s" % mpath_name
        pid = dt.dt_stress_background(of="/dev/mapper/%s" % mpath_name,
                                      log_file=dt_log_file,
                                      verbose=0)
        if not pid:
            _print("FAIL: Could not start DT I/O stress on %s" % mpath_name)
            _clean_up()
            sys.exit(1)
        dt_pid_2_info[pid] = {"log": dt_log_file,
                              "exit_code": None}

    for dt_pid in dt_pid_2_info.keys():
        # option for non blocking, in this case we will block. os.WNOHANG
        flag_dt_done, exit_status = os.waitpid(dt_pid, 0)
        dt_pid_2_info[dt_pid]["exit_code"] = exit_status

    dt_fail = False
    for dt_pid in dt_pid_2_info.keys():
        if dt_pid_2_info[dt_pid]["exit_code"] != 0:
            _print("FAIL: DT I/O stress PID %s failed with quit code %s" %
                   (dt_pid, dt_pid_2_info[dt_pid]["exit_code"]))
            run("cat %s" % dt_pid_2_info[dt_pid]["log"])
            dt_fail = True

    if not dt_fail:
        _print("INFO: All I/O stress tests PASS")

    _clean_up()
    if not log.check_all() or dt_fail:
        _print("FAIL: detected error on LogChecker or IO error")
        return False

    _print("PASS: All I/O stress tests")
    return True


def main():
    """
    """
    PASS = 0
    FAIL = 1

    parser = argparse.ArgumentParser(
        description='add new num')
    parser.add_argument("--mpath-name", "-m", required=False, dest="mpath_name",
                        help="Name of multipath device to use (We use it to create new device based on it)",
                        metavar="mpath")
    parser.add_argument("--lun-count", required=False, dest="lun_count", default=1,
                        help="How many LUN should be created", type=int, metavar="count")
    parser.add_argument("--lun-size", required=False, dest="lun_size", default="2GiB",
                        help="What size should be the new LUN", metavar="size. Eg: 2GiB")

    args = parser.parse_args()

    mpath_name_list = []
    if args.mpath_name:
        mpath_name_list.append(args.mpath_name)
    lun_count = args.lun_count
    lun_size = args.lun_size

    # If not mpath was specified search for them on the system
    if not mpath_name_list:
        choosen_devs = sanmgmt.choose_mpaths()
        if not choosen_devs:
            _print("FAIL: Could not find any multipath device to use as base LUN")
            sys.exit(FAIL)
        mpath_name_list = list(choosen_devs.keys())

    error = 0
    for mpath_name in mpath_name_list:
        if not execute_test(mpath_name, lun_count, lun_size):
            error += 1

    if error:
        sys.exit(FAIL)
    sys.exit(PASS)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.linux import load_module, unload_module, is_module_loaded
from libsan.host.cmdline import run
from stqe.host.atomic_run import atomic_run, parse_ret
from os import environ


def kvdo():
    # Tries to load/unload kvdo kernel module.
    errors = []
    arguments = []

    # clean up modules first, to be sure (kvdo blocks uds)
    for module in ['kvdo', 'uds']:
        ret, data = run(cmd="lsmod | grep %s" % module, return_output=True)
        if data != "":
            arguments += [{'message': "Unloading module '%s'." % module, 'command': unload_module,
                           'module_name': module}]

    arguments += [
        {'message': "Loading module 'kvdo'.", 'command': load_module, 'module': "kvdo"},
        {'message': "Checking if 'kvdo' is loaded.", 'command': is_module_loaded, 'module_name': "kvdo"},
        # kvdo requires uds
        {'message': "Checking if 'uds' is loaded.", 'command': is_module_loaded, 'module_name': "kvdo"},
        {'message': "Unloading module 'kvdo'.", 'command': unload_module, 'module_name': "kvdo"},
        {'message': "Unloading module 'uds'.", 'command': unload_module, 'module_name': "uds"}
    ]

    for argument in arguments:
        atomic_run(errors=errors,
                   **argument)

    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = kvdo()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config, get_data_from_script_output
from os import environ
from stqe.host.persistent_vars import read_var


def system_read_cache_pct_update_success():
    errors = []

    sys = read_var("LSM_SYS_ID")

    read_pct = int(read_var("LSM_SYSTEM_READ_PCT"))

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        ret, data = atomic_run("Listing systems to backup read percentage",
                               command=lsm.list,
                               lsm_type="SYSTEMS",
                               return_output=True,
                               script=True,
                               errors=errors)
        pct_before = get_data_from_script_output(data)[sys]['Read Cache Percentage']

        atomic_run("Changing system read pct to %s with protocol %s" % (read_pct, config['protocol']),
                   command=lsm.system_read_cache_pct_update,
                   sys=sys,
                   read_pct=read_pct,
                   errors=errors)

        ret, data = atomic_run("Listing systems to check updated read percentage",
                               command=lsm.list,
                               lsm_type="SYSTEMS",
                               return_output=True,
                               script=True,
                               errors=errors)
        pct_after = int(get_data_from_script_output(data)[sys]['Read Cache Percentage'])
        if pct_after != read_pct:
            msg = "FAIL: System cache read percentage did not change, should be %s, got %s" % (read_pct, pct_after)
            print(msg)
            errors.append(msg)

        atomic_run("Returning system read pct to former value with protocol %s" % config['protocol'],
                   command=lsm.system_read_cache_pct_update,
                   sys=sys,
                   read_pct=pct_before,
                   errors=errors)

    return errors


def system_read_cache_pct_update_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {'message': "Trying to fail changing system read pct without path with protocol %s" % config['protocol'],
             'command': lsm.system_read_cache_pct_update},
            {'message': "Trying to fail changing system read pct with wrong system with protocol %s" %
                        config['protocol'], 'command': lsm.system_read_cache_pct_update, 'sys': "WRONG"}
        ]
        for argument in arguments:
            atomic_run(expected_ret=2,
                       errors=errors,
                       **argument)
    return errors


if __name__ == "__main__":
    if int(environ['fmf_tier']) == 1:
        errs = system_read_cache_pct_update_success()
    if int(environ['fmf_tier']) == 2:
        errs = system_read_cache_pct_update_fail()
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.lsm import LibStorageMgmt
from stqe.host.lsm import yield_lsm_config
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var
from os import environ


def create_access_group():
    print("INFO: Creating AG.")
    errors = []

    ag_name = read_var("LSM_AG_NAME") + environ['fmf_init'].upper()
    sys_id = read_var("LSM_SYS_ID")
    init_id = read_var("LSM_INIT_ID" + environ['fmf_init'].upper())

    config = list(yield_lsm_config())[0]
    if 'smis' in config['protocol']:
        print("smis protocol does not support ag creation on EMC. skipping.")
        return errors
    lsm = LibStorageMgmt(disable_check=True, **config)

    ret, data = atomic_run("Creating AG '%s'." % ag_name,
                           command=lsm.access_group_create,
                           name=ag_name,
                           init=init_id,
                           sys=sys_id,
                           return_output=True,
                           errors=errors)

    for line in data.splitlines():
        if ag_name in line:
            ag_id = line.split()[0].strip()
            atomic_run("Writing var LSM_AG_ID" + environ['fmf_init'].upper(),
                       command=write_var,
                       var={"LSM_AG_ID" + environ['fmf_init'].upper(): ag_id},
                       errors=errors)

    return errors


if __name__ == "__main__":
    errs = create_access_group()
    exit(parse_ret(errs))



#!/usr/bin/python

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.cmdline import run
from libsan.host.loopdev import create_loopdev, delete_loopdev
import libsan.host.lvm as lvm
import libsan.host.dm as dm
import libsan.host.linux as linux
import libsan.host.snapper as snapper
import stqe.host.tc
import sys

#
# Bugs related:
#   BZ1365555 - Revert workaround for issues with snapper and btrfs subvolume labels
#

TestObj = None

loop_dev = None

vg_name = "vgtest"
pool_name = "test_pool"
thin1_name = "thin1"
brtfs_name = "brtfs_lv"
mnt_point_dev = "/mnt/snapper_test"
cfg_name = "bugtest"


def create_test_lv(filesystem):
    if not filesystem:
        return None

    # On brtfs we should not use thinp
    if filesystem != "btrfs":
        if run("lvcreate --thinpool %s -L 800M %s" % (pool_name, vg_name)) != 0:
            print("FAIL: Could not create %s - %s" % (pool_name, filesystem))
            return None
        if run("lvcreate -V 1G -T %s/%s -n %s" % (vg_name, pool_name, thin1_name)) != 0:
            print("FAIL: Could not create %s - %s" % (thin1_name, filesystem))
            return None

        return thin1_name
    else:
        if run("lvcreate -L 500M  -n %s %s" % (brtfs_name, vg_name)) != 0:
            print("FAIL: Could not create %s - %s" % (brtfs_name, filesystem))
            return None

        return brtfs_name


def start_test(filesystem, snapper_debug_mode):
    global TestObj

    print(80 * "#")
    print("INFO: Starting test on using snapper on %s debug mode:%s" % (filesystem, snapper_debug_mode))
    print(80 * "#")

    if snapper_debug_mode:
        # Wait 60 seconds so snapperd terminates
        linux.sleep(60)
        if not snapper.snapper_enable_debug_mode():
            TestObj.tfail("Could not enable snapper debug on FS(%s)" % filesystem)
            return False

    # create loop device of 900M
    global loop_dev
    loop_dev = create_loopdev(size=900)
    if not loop_dev:
        TestObj.tfail("Could not create loop device")
        return False

    if not lvm.vg_create(vg_name, loop_dev):
        TestObj.tfail("Could not create VG \"%s\"" % vg_name)
        return False

    print("INFO: Creating LVs")
    test_lv = create_test_lv(filesystem)
    if not test_lv:
        TestObj.tfail("Could not LV for FS(%s) with snapper_debug_mode=%s" % (filesystem, snapper_debug_mode))
        return False

    dm.dm_show_table()

    test_device = "/dev/mapper/%s-%s" % (vg_name, test_lv)
    print("INFO:Going to create FS(%s) on %s" % (filesystem, test_device))
    fs_option = ""
    if filesystem == "btrfs":
        # use mixed option as device is not big. more details see:
        # https://bugzilla.redhat.com/show_bug.cgi?id=1374158#c19
        fs_option = "--mixed"
    if run("mkfs.%s %s %s" % (filesystem, fs_option, test_device)) != 0:
        TestObj.tfail("Could not create FS(%s) on %s with snapper_debug_mode=%s"
                      % (filesystem, test_device, snapper_debug_mode))
        return False

    if not linux.mkdir(mnt_point_dev):
        TestObj.tfail("Could not create directory %s - FS(%s) with snapper_debug_mode=%s" % (
            mnt_point_dev, filesystem, snapper_debug_mode))
        return False

    if not linux.mount(test_device, mnt_point_dev):
        TestObj.tfail("Could not mount %s - FS(%s) with snapper_debug_mode=%s"
                      % (mnt_point_dev, filesystem, snapper_debug_mode))
        return False

    lvm.lv_show()

    print("INFO: Going create snapper config %s" % cfg_name)
    snap_type = "lvm(%s)" % filesystem
    if filesystem == "btrfs":
        snap_type = "btrfs"
    if not snapper.snapper_create_config(cfg_name, snap_type, mnt_point_dev):
        TestObj.tfail("Could not create snapper config - FS(%s) with snapper_debug_mode=%s"
                      % (filesystem, snapper_debug_mode))
        return False

    run("ps -ef | grep snapper")

    if not snapper.snapper_list_configs():
        TestObj.tfail("Could not list config FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    # list current snapshots
    # do not expect any snaphost to be listed
    if not snapper.snapper_list(cfg_name):
        TestObj.tfail("Could not list snapshots FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    print("INFO: Creating single snapshot")
    pre_num = snapper.snapper_create(cfg_name, "single")
    if not pre_num:
        TestObj.tfail("Could not create SINGLE snapshot FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    if not linux.mkdir("%s/dir_0" % mnt_point_dev):
        TestObj.tfail("Could not create directory on %s - FS(%s) with snapper_debug_mode=%s" % (
            mnt_point_dev, filesystem, snapper_debug_mode))
        return False
    if run("touch %s/dir_0/file_0" % mnt_point_dev) != 0:
        TestObj.tfail("Could not ecreate file FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    print("INFO: Creating pre snapshot")
    pre_num = snapper.snapper_create(cfg_name, "pre")
    if not pre_num:
        TestObj.tfail("Could not create PRE snapshot FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    if not snapper.snapper_list(cfg_name):
        TestObj.tfail("Could not list snapshots FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    print("INFO: Creating data on %s" % mnt_point_dev)
    if not linux.mkdir("%s/dir_1" % mnt_point_dev):
        TestObj.tfail("Could not create directory on %s - FS(%s) with snapper_debug_mode=%s" % (
            mnt_point_dev, filesystem, snapper_debug_mode))
        return False
    if run("touch %s/dir_1/file_1" % mnt_point_dev) != 0:
        TestObj.tfail("Could not create file FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False
    if run("echo \"going to create snapshot\" >> %s/dir_0/file_0" % mnt_point_dev) != 0:
        TestObj.tfail("Could not edit file FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    print("INFO: Creating post snapshot")
    post_num = snapper.snapper_create(cfg_name, "post", pre_num=pre_num)
    if not post_num:
        TestObj.tfail("Could not create POST snapshot FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    if not snapper.snapper_list(cfg_name):
        TestObj.tfail("Could not list snapshots FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    if not snapper.snapper_status(cfg_name, pre_num, post_num):
        TestObj.tfail("Could not get status for snapshots FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    if not snapper.snapper_diff(cfg_name, pre_num, post_num):
        TestObj.tfail("Could not get diff for snapshots FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    print("INFO: Deleting post snapshot")
    if not snapper.snapper_delete(cfg_name, post_num):
        TestObj.tfail("Could not delete POST snapshot FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    if not snapper.snapper_list(cfg_name):
        TestObj.tfail("Could not list snapshots FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    print("INFO: Deleting pre snapshot")
    if not snapper.snapper_delete(cfg_name, pre_num):
        TestObj.tfail("Could not delete PRE snapshot FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    print("INFO: Deleting config")
    if not snapper.snapper_delete_config(cfg_name):
        TestObj.tfail("Could not delete config FS(%s) on %s with snapper_debug_mode=%s" % (
            filesystem, test_device, snapper_debug_mode))
        return False

    # Wait 60 seconds so snapperd terminates
    linux.sleep(60)
    if snapper_debug_mode:
        if not snapper.snapper_disable_debug_mode():
            TestObj.tfail("Could not disable snapper debug on FS(%s)" % filesystem)
            return False

    print(80 * "#")
    TestObj.tpass("PASS: Test on FS %s and snapper_debug_mode=%s" % (filesystem, snapper_debug_mode))
    print(80 * "#")

    return True


def _clean_up():
    global vg_name, loop_dev

    snapper.snapper_delete_config(cfg_name)

    linux.umount(mnt_point_dev)

    # make sure any failed device is removed
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail("Could not delete VG \"%s\"" % vg_name)

    if loop_dev:
        if not lvm.pv_remove(loop_dev):
            TestObj.tfail("Could not delete PV \"%s\"" % loop_dev)
        delete_loopdev(loop_dev)


def execute_test():
    global TestObj

    os_version = linux.dist_ver()
    os_name = linux.dist_name()

    if os_name == "RHEL" and os_version < 7:
        TestObj.tskip("Test is not supported on %s-%s" % (os_name, os_version))
        return

    linux.install_package("snapper")

    filesystems = ["ext4", "xfs"]
    # btrfs is depreciated in RHEL 7.4, but should remain accessible in RHEL 7 series
    # also btrfs was purposely removed from Pegas Kernel (RHEL ALT)
    ret, kernel = run(cmd='uname -r', return_output=True)
    if ret:
        print("WARN: Could not get kernele version, running without btrfs.")
    else:
        version = kernel.split("-")[0].split(".")
        # FIXME: add check for RHEL < 8
        if int(version[0]) <= 4 and int(version[1]) <= 11:
            filesystems.append("btrfs")
    # test using snapperd with and without debug
    dbg_modes = [False, True]

    for fs in filesystems:
        for dbg_mode in dbg_modes:
            _clean_up()

            if not start_test(fs, dbg_mode):
                log_name = "snapper_%s_%s.log" % (fs, dbg_mode)
                run("mv /var/log/snapper.log %s" % log_name)
                TestObj.log_submit(log_name)

    _clean_up()
    return


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("snapper")
    linux.install_package("lvm2")

    execute_test()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from libsan.host.vdo import VDOStats
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, read_env
from stqe.host.vdo import is_block_device
from libsan.host.cmdline import run
from shutil import copy
from libsan.host.linux import compare_version
import libsan.misc.time as time


def single_copy(vdostats, device, vdo_name, fs_dir, file, duration):
    error = []
    file_name = "vdo_test.file"
    expected_ret = 0
    if compare_version("vdo", "6.2.0.0", "0") and not compare_version("vdo", "6.2.0.71", "71-4"):
        # Hit vdostats bug
        expected_ret = 1

    arguments = [
        {'message': "Copying file %s to %s/%s" % (file, fs_dir, file_name), 'command': copy, 'src': "%s" % file,
         'dst': "%s/%s" % (fs_dir, file_name)},
        {'message': "Umounting filesystem", 'command': run, 'cmd': "umount %s" % fs_dir},
        {'message': "Mounting XFS filesystem with discard enabled", 'command': run,
         'cmd': "mount -o discard %s %s" % (device, fs_dir)},
        {'message': "Removing file %s/%s" % (fs_dir, file_name), 'command': run,
         'cmd': "rm -f %s/%s" % (fs_dir, file_name)},
        {'message': "Umounting filesystem", 'command': run, 'cmd': "umount %s" % fs_dir},
        {'message': "Checking stats of vdo %s" % vdo_name, 'command': vdostats.stats, 'expected_ret': expected_ret},
        {'message': "Mounting XFS filesystem with discard enabled", 'command': run,
         'cmd': "mount -o discard %s %s" % (device, fs_dir)}
    ]

    start_time = time.get_time(in_seconds=True)
    for argument in arguments:
        atomic_run(errors=error,
                   **argument)
    end_time = time.get_time(in_seconds=True)
    total_time = time.time_2_sec(end_time - start_time)
    duration.append(total_time)

    return duration, error


def repeated_copy(repeat_count):
    errors = []
    vdostats = VDOStats()
    vdo_name = read_env('fmf_vdo_name')
    device = read_var("VDO_DEVICE")
    fs_dir = read_var("FS_DIR")
    file = read_var("DATA_FILE")

    ret = is_block_device(device)
    if ret != True:
        errors.append(ret)
        return errors

    duration = list()
    i = 0
    while i < repeat_count:
        i += 1
        print("############# Running iteration %s #############" % (i))
        duration, ret = single_copy(vdostats, device, vdo_name, fs_dir, file, duration)
        errors += ret
        print("######### This iteration took %s  #########" % time.sec_2_time(duration[-1]))

        # Reduce iteration count in case the iteration took too long.
        try:
            max_runtime = read_env("fmf_max_runtime")
            avg_duration = sum(duration) / len(duration)
            count = int(max_runtime * 60 / avg_duration)
            if count < repeat_count:
                print("INFO: Changing total amount of iterations from %s to %s to avoid timeout." %
                      (repeat_count, count))
                repeat_count = count
        except KeyError:
            pass

    print("\n####### SUMMARY of %s iterations ########" % (i))
    print("######### Average duration: %s ##########" % time.sec_2_time(int(sum(duration) / len(duration))))
    print("######### Minimum duration: %s ##########" % time.sec_2_time(min(duration)))
    print("######### Maximum duration: %s ##########\n" % time.sec_2_time(max(duration)))
    return errors


if __name__ == "__main__":
    if read_env('fmf_tier') == 1:
        errs = repeated_copy(read_env('fmf_repeat_count'))
    if read_env('fmf_tier') == 2:
        errs = repeated_copy(read_env('fmf_repeat_count'))
    exit(parse_ret(errs))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var
from libsan.host.lio import TargetCLI


def loopback_cleanup():
    errors = []
    wwn = read_var("LOOPBACK_WWN")
    lun = "lun" + str(read_var("LOOPBACK_LUN"))
    target = TargetCLI(path="/loopback/%s/luns" % wwn)

    atomic_run("Deleting lun: %s" % lun,
               lun=lun,
               command=target.delete,
               errors=errors
               )

    target.path = "/loopback"

    atomic_run("Deleting loopback: %s" % wwn,
               wwn=wwn,
               command=target.delete,
               errors=errors
               )

    return errors


if __name__ == "__main__":
    errs = loopback_cleanup()
    exit(parse_ret(errs))
